unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Menus, FileCtrl, Preferences, registry, ExtCtrls, ComCtrls, jpeg,
  PackageFormat;

type
  TMainForm = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    Help2: TMenuItem;
    N1: TMenuItem;
    Options1: TMenuItem;
    Preferences1: TMenuItem;
    BackgroundPanel: TPanel;
    PackageGroupBox: TGroupBox;
    AvailPackages: TFileListBox;
    PreviewBox: TGroupBox;
    GroupBox3: TGroupBox;
    GroupList: TListBox;
    GroupBox1: TGroupBox;
    PackageVersion: TLabel;
    GroupCount: TLabel;
    TextureCount: TLabel;
    FileSize: TLabel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    TexturePanel: TPanel;
    PreviewUpDown: TUpDown;
    Panel0: TPanel;
    Image0: TImage;
    PreviewDesc0: TLabel;
    Panel1: TPanel;
    Image1: TImage;
    PreviewDesc1: TLabel;
    Panel2: TPanel;
    Image2: TImage;
    PreviewDesc2: TLabel;
    Panel3: TPanel;
    Image3: TImage;
    PreviewDesc3: TLabel;
    Panel4: TPanel;
    Image4: TImage;
    PreviewDesc4: TLabel;
    Panel5: TPanel;
    Image5: TImage;
    PreviewDesc5: TLabel;
    Panel6: TPanel;
    Image6: TImage;
    PreviewDesc6: TLabel;
    Panel7: TPanel;
    Image7: TImage;
    PreviewDesc7: TLabel;
    Panel8: TPanel;
    Image8: TImage;
    PreviewDesc8: TLabel;
    TexturePopup: TPopupMenu;
    ExporttoBMP1: TMenuItem;
    ExporttoJPEG1: TMenuItem;
    N2: TMenuItem;
    ShowLargestMipMap1: TMenuItem;
    TiledPreview1: TMenuItem;
    pnlStatus: TPanel;
    Bevel1: TBevel;
    edbSearch: TEdit;
    btnSearch: TButton;
    Button1: TButton;
    procedure Exit1Click(Sender: TObject);
    procedure Preferences1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure AvailPackagesClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure GroupListClick(Sender: TObject);
    procedure PreviewUpDownClick(Sender: TObject; Button: TUDBtnType);
    procedure ExporttoBMP1Click(Sender: TObject);
    procedure Image0MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image2MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image3MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image4MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image5MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image6MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image7MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image8MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ExporttoJPEG1Click(Sender: TObject);
    procedure TiledPreview1Click(Sender: TObject);
    procedure ShowLargestMipMap1Click(Sender: TObject);
    procedure btnSearchClick(Sender: TObject);
  private
    procedure UpdatePackageList;
    procedure DrawPreviews(const TextureIndex : integer);
    procedure ShowHint(Sender : Tobject);
    procedure MyException(Sender: TObject; E: Exception);
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  MainForm   : TMainForm;
  UTDir      : string;
  UTXDir     : string;
  UTPackage  : TUTPackage; //Object of TexturePackage
  IsLoaded   : Boolean;    //Package successfully loaded ?
  GroupIndex : integer;    //Index of current group
  SelTexture : integer;    //Index of selected-texture when using the popup


implementation

uses AboutDlg, TiledPreviewDlg, FullPreviewDlg, ErrorBox;

{$R *.DFM}

// Initialise the hint handling routine
procedure TMainForm.ShowHint(Sender: TObject);
begin
  PnlStatus.Caption := '   ' + application.Hint;
end;

// Initialise the exception handling routine
procedure TMainForm.MyException(Sender: TObject; E: Exception);
begin
  frmError.lblError.caption := E.Message;
  frmError.show;
end;

////////////////////////////////////////////////////////////////////////////////
//Open dialog and init-stuff
procedure TMainForm.Exit1Click(Sender: TObject);
begin
  //Finish objects
  UTPackage.Free;
  Close;
end;

procedure TMainForm.Preferences1Click(Sender: TObject);
begin
  PreferenceForm := TPreferenceForm.Create(Application);
  PreferenceForm.ShowModal;   //Show Dialog
  PreferenceForm.Free;

  UpdatePackageList;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  Reg        : TRegistry;
  RegSuccess : Boolean;
begin
  // Setup custom exception handler
  Application.OnException := MyException;
  btnSearch.Caption:='Package &Search';
  Application.OnHint := ShowHint;
  //Test if registry-values are correct
  Reg := TRegistry.Create;
  Reg.RootKey := HKEY_LOCAL_MACHINE;
  RegSuccess := Reg.OpenKey('SOFTWARE\Unreal Technology\Installed Apps\UnrealTournament', FALSE);
  if RegSuccess = TRUE then
    UTDir   := Reg.ReadString('Folder') + '\'
  else
    MessageDlg('Your version of UnrealTournament is not properly installed. '
             + 'Ignore all following error-messages, open the preference-dialog '
             + 'and enter the correct folder (without "\" at the end). '
             + 'Then restart the app.' , mtError, [mbOK], 0);

  Reg.Free;

  //Set Texture-Directory
  UpdatePackageList;

  //Disable scroll-bar at startup
  PreviewUpDown.Enabled := FALSE;
end;

procedure TMainForm.UpdatePackageList;
begin
  //later use of different folders
  UTXDir  := UTDir + 'Textures\'; //Standard Texture-Folder
  AvailPackages.Directory := UTXDir;
  AvailPackages.Update;
end;
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//TexturePreview
procedure TMainForm.AvailPackagesClick(Sender: TObject);
var
  i     : integer;
begin
  //Setup everything
  GroupList.Clear;

  //Free PackageObject for use
  UTPackage.Free;

  UTPackage := TUTPackage.Create;

  IsLoaded := UTPackage.Load(AvailPackages.FileName);
  if (IsLoaded = FALSE) then
  begin
    UTPackage.Free;
    MessageDlg('Package ' + AvailPackages.FileName + ' could not be loaded', mtError, [mbOK], 0);
  end;

  for i:=0 to UTPackage.GroupCount-1 do
    GroupList.Items.Add(UTPackage.Group[i].Name);
  //automatically show [ALL] textures
  GroupList.ItemIndex := GroupList.Items.Count-1;
  GroupListClick(Application);;
  GroupList.Sorted := TRUE;

  //Print infos on Package to screen
  PackageVersion.Caption := IntToStr(UTPackage.PkgVer);
  GroupCount.Caption := IntToStr(UTPackage.GroupCount);
  TextureCount.Caption := IntToStr(UTPackage.TextureCount);
  FileSize.Caption := FloatToStrF(UTPackage.PackageSize/(1024*1024), ffFixed, 2, 2) + ' MB';

  //Activate Scrollbar
  PreviewUpDown.Enabled := TRUE;
end;

procedure TMainForm.About1Click(Sender: TObject);
begin
  AboutBox := TAboutBox.Create(Application);
  AboutBox.ShowModal;
  AboutBox.Free;
end;

procedure TMainForm.GroupListClick(Sender: TObject);
var
  i,j       : integer;
begin
  //RePaint GroupList (looks nicer)
  GroupList.Sorted := TRUE;
  GroupList.RePaint;

  //Display previews
  for i:=0 to GroupList.Items.Count-1 do
    if (GroupList.Selected[i] = TRUE) then
    begin
      for j:=0 to UTPackage.GroupCount-1 do
        if (GroupList.Items.Strings[i] = UTPackage.Group[j].Name) then
        begin
          //Init scroll-bar
          PreviewUpDown.Min := 0;
          PreviewUpDown.Max := UTPackage.Group[j].Entries.Count;
          PreviewUpDown.Increment := 9;
          PreviewUpDown.Position := 0;

          GroupIndex := j;

          DrawPreviews(PreviewUpDown.Position);

          break;
      end;
    end;
end;

procedure TMainForm.PreviewUpDownClick(Sender: TObject;
  Button: TUDBtnType);
begin
  if (PreviewUpDown.Position >= UTPackage.Group[GroupIndex].Entries.Count) then
    PreviewUpDown.Position := 0;
  DrawPreviews(PreviewUpDown.Position);
end;

procedure TMainForm.DrawPreviews;
var
  Position : integer;
begin
  Position := TextureIndex;

  //Draw previews
  begin
    if (Position < UTPackage.Group[GroupIndex].Entries.Count) then
    begin
      UTPackage.LoadNormalTexture(UTPackage.Group[GroupIndex].Entries.Strings[Position], 128);
      Image0.Picture.Bitmap := UTPackage.NormalTexture.B;
      Image0.RePaint;
      Image0.Show;
      PreviewDesc0.Caption := '(' + IntToStr(Position+1) + '/' + IntToStr(UTPackage.Group[GroupIndex].Entries.Count) + ')'
                                  + ' - ' + UTPackage.NormalTexture.TextureName;
      PreviewDesc0.RePaint;
      Position := Position + 1;
    end
    else
    begin
      Image0.Hide;
      PreviewDesc0.Caption := '';
    end;
    if (Position < UTPackage.Group[GroupIndex].Entries.Count) then
    begin
      UTPackage.LoadNormalTexture(UTPackage.Group[GroupIndex].Entries.Strings[Position], 128);
      Image1.Picture.Bitmap := UTPackage.NormalTexture.B;
      Image1.Show;
      Image1.RePaint;
      PreviewDesc1.Caption := '(' + IntToStr(Position+1) + '/' + IntToStr(UTPackage.Group[GroupIndex].Entries.Count) + ')'
                                 + ' - ' + UTPackage.NormalTexture.TextureName;
      PreviewDesc1.RePaint;
      Position := Position + 1;
    end
    else
    begin
      Image1.Hide;
      PreviewDesc1.Caption := '';
    end;
    if (Position < UTPackage.Group[GroupIndex].Entries.Count) then
    begin
      UTPackage.LoadNormalTexture(UTPackage.Group[GroupIndex].Entries.Strings[Position], 128);
      Image2.Picture.Bitmap := UTPackage.NormalTexture.B;
      Image2.Show;
      Image2.RePaint;
      PreviewDesc2.Caption := '(' + IntToStr(Position+1) + '/' + IntToStr(UTPackage.Group[GroupIndex].Entries.Count) + ')'
                                  + ' - ' + UTPackage.NormalTexture.TextureName;
      PreviewDesc2.RePaint;
      Position := Position + 1;
    end
    else
    begin
      Image2.Hide;
      PreviewDesc2.Caption := '';
    end;
    if (Position < UTPackage.Group[GroupIndex].Entries.Count) then
    begin
      UTPackage.LoadNormalTexture(UTPackage.Group[GroupIndex].Entries.Strings[Position], 128);
      Image3.Picture.Bitmap := UTPackage.NormalTexture.B;
      Image3.Show;
      Image3.RePaint;
      PreviewDesc3.Caption := '(' + IntToStr(Position+1) + '/' + IntToStr(UTPackage.Group[GroupIndex].Entries.Count) + ')'
                                 + ' - ' + UTPackage.NormalTexture.TextureName;
      PreviewDesc3.RePaint;
      Position := Position + 1;
    end
    else
    begin
      Image3.Hide;
      PreviewDesc3.Caption := '';
    end;
    if (Position < UTPackage.Group[GroupIndex].Entries.Count) then
    begin
      UTPackage.LoadNormalTexture(UTPackage.Group[GroupIndex].Entries.Strings[Position], 128);
      Image4.Picture.Bitmap := UTPackage.NormalTexture.B;
      Image4.Show;
      Image4.RePaint;
      PreviewDesc4.Caption := '(' + IntToStr(Position+1) + '/' + IntToStr(UTPackage.Group[GroupIndex].Entries.Count) + ')'
                                  + ' - ' + UTPackage.NormalTexture.TextureName;
      PreviewDesc4.RePaint;
      Position := Position + 1;
    end
    else
    begin
      Image4.Hide;
      PreviewDesc4.Caption := '';
    end;
    if (Position < UTPackage.Group[GroupIndex].Entries.Count) then
    begin
      UTPackage.LoadNormalTexture(UTPackage.Group[GroupIndex].Entries.Strings[Position], 128);
      Image5.Picture.Bitmap := UTPackage.NormalTexture.B;
      Image5.Show;
      Image5.RePaint;
      PreviewDesc5.Caption := '(' + IntToStr(Position+1) + '/' + IntToStr(UTPackage.Group[GroupIndex].Entries.Count) + ')'
                                  + ' - ' + UTPackage.NormalTexture.TextureName;
      PreviewDesc5.RePaint;
      Position := Position + 1;
    end
    else
    begin
      Image5.Hide;
      PreviewDesc5.Caption := '';
    end;
    if (Position < UTPackage.Group[GroupIndex].Entries.Count) then
    begin
      UTPackage.LoadNormalTexture(UTPackage.Group[GroupIndex].Entries.Strings[Position], 128);
      Image6.Picture.Bitmap := UTPackage.NormalTexture.B;
      Image6.Show;
      Image6.RePaint;
      PreviewDesc6.Caption := '(' + IntToStr(Position+1) + '/' + IntToStr(UTPackage.Group[GroupIndex].Entries.Count) + ')'
                                  + ' - ' + UTPackage.NormalTexture.TextureName;
      PreviewDesc6.RePaint;
      Position := Position + 1;
    end
    else
    begin
      Image6.Hide;
      PreviewDesc6.Caption := '';
    end;
    if (Position < UTPackage.Group[GroupIndex].Entries.Count) then
    begin
      UTPackage.LoadNormalTexture(UTPackage.Group[GroupIndex].Entries.Strings[Position], 128);
      Image7.Picture.Bitmap := UTPackage.NormalTexture.B;
      Image7.Show;
      Image7.RePaint;
      PreviewDesc7.Caption := '(' + IntToStr(Position+1) + '/' + IntToStr(UTPackage.Group[GroupIndex].Entries.Count) + ')'
                                  + ' - ' + UTPackage.NormalTexture.TextureName;
      PreviewDesc7.RePaint;
      Position := Position + 1;
    end
    else
    begin
      Image7.Hide;
      PreviewDesc7.Caption := '';
    end;
    if (Position < UTPackage.Group[GroupIndex].Entries.Count) then
    begin
      UTPackage.LoadNormalTexture(UTPackage.Group[GroupIndex].Entries.Strings[Position], 128);
      Image8.Picture.Bitmap := UTPackage.NormalTexture.B;
      Image8.Show;
      Image8.RePaint;
      PreviewDesc8.Caption := '(' + IntToStr(Position+1) + '/' + IntToStr(UTPackage.Group[GroupIndex].Entries.Count) + ')'
                                + ' - ' + UTPackage.NormalTexture.TextureName;
      PreviewDesc8.RePaint;
      //Position := Position + 1; //not necessary
    end
    else
    begin
      Image8.Hide;
      PreviewDesc8.Caption := '';
    end;
  end;
end;
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//Texture Popup

////////////////////////////////////
//Get TextureIndex of actual texture
procedure TMainForm.Image0MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbRight) then
    SelTexture := PreviewUpDown.Position + 0;
end;
procedure TMainForm.Image1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbRight) then
    SelTexture := PreviewUpDown.Position + 1;
end;
procedure TMainForm.Image2MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbRight) then
    SelTexture := PreviewUpDown.Position + 2;
end;
procedure TMainForm.Image3MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbRight) then
    SelTexture := PreviewUpDown.Position + 3;
end;
procedure TMainForm.Image4MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbRight) then
    SelTexture := PreviewUpDown.Position + 4;
end;
procedure TMainForm.Image5MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbRight) then
    SelTexture := PreviewUpDown.Position + 5;
end;
procedure TMainForm.Image6MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbRight) then
    SelTexture := PreviewUpDown.Position + 6;
end;
procedure TMainForm.Image7MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbRight) then
    SelTexture := PreviewUpDown.Position + 7;
end;
procedure TMainForm.Image8MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbRight) then
    SelTexture := PreviewUpDown.Position + 8;
end;

///////////////
//Export to BMP
procedure TMainForm.ExporttoBMP1Click(Sender: TObject);
var
  SaveBMP : TBitmap;
begin
  /////////////////
  //NORMAL TEXTURES
  UTPackage.LoadNormalTexture(UTPackage.Group[GroupIndex].Entries.Strings[SelTexture], 512); //Maximum supported texture size is 512*512
  SaveBMP := UTPackage.NormalTexture.B;
  SaveBMP.SaveToFile(UTDir + '\' + UTPackage.Group[GroupIndex].Entries.Strings[SelTexture] + '.bmp');
end;
//Export to JPEG
procedure TMainForm.ExporttoJPEG1Click(Sender: TObject);
var
  SaveJPEG : TJPEGImage;
begin
  /////////////////
  //NORMAL TEXTURES
  UTPackage.LoadNormalTexture(UTPackage.Group[GroupIndex].Entries.Strings[SelTexture], 512); //Maximum supported texture size is 512*512
  SaveJPEG := TJPEGImage.Create;
  SaveJPEG.Assign(UTPackage.NormalTexture.B);
  SaveJPEG.CompressionQuality := 75;
  SaveJPEG.Compress;
  SaveJPEG.SaveToFile(UTDir + '\' + UTPackage.Group[GroupIndex].Entries.Strings[SelTexture] + '.jpg');
  SaveJPEG.Free;
end;

//////////////
//TiledPreview
procedure TMainForm.TiledPreview1Click(Sender: TObject);
begin
  /////////////////
  //NORMAL TEXTURES
  UTPackage.LoadNormalTexture(UTPackage.Group[GroupIndex].Entries.Strings[SelTexture], 512); //Maximum supported texture size is 512*512

  TiledPreviewForm := TTiledPreviewForm.Create(Application);
  TiledPreviewForm.ShowModal(UTPackage.NormalTexture.B);
end;
/////////////
//Full preview
procedure TMainForm.ShowLargestMipMap1Click(Sender: TObject);
begin
  /////////////////
  //NORMAL TEXTURES
  UTPackage.LoadNormalTexture(UTPackage.Group[GroupIndex].Entries.Strings[SelTexture], 512); //Maximum supported texture size is 512*512

  FullPreviewForm := TFullPreviewForm.Create(Application);
  FullPreviewForm.ShowModal(UTPackage.NormalTexture.B);
end;
////////////////////////////////////////////////////////////////////////////

procedure TMainForm.btnSearchClick(Sender: TObject);
begin
  if edbSearch.text = '' then
  begin
    btnSearch.Caption:='Package &Search';
    AvailPackages.Mask := '*.u*';
    AvailPackages.Update;
  end
 else
  begin
    AvailPackages.Mask := '*'+EdbSearch.text+'*.u*';
    AvailPackages.Update;
    btnSearch.Caption:='&Show All';
    EdbSearch.text:='';
  end;
end;

end.
