////////////////////////////////////////////////////////////////////////////////
//
// NormalTexture-Unit:
//
// Reads a normal texture from any UTPackage into a TBitmap-object,
// requieres a copy of the HeaderTables and the file-object as parameters.
//
// Includes: - TMipMap (contains unconverted texture and the palette)
//           - TTexture (general properties and the converted texture)
//
// (c) 2000, by Jesco Topp
////////////////////////////////////////////////////////////////////////////////
unit NormalTexture;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,
  PkgAux;

////////////////////////////////////////////////////////////////////////////////

type
  TMipMap = Class(TObject)
    WidthOffset    : DWORD; //(only if PkgVer >= 63)
    MipMapSize     : DWORD;
    MipMapData     : array [0..262143] of BYTE; //max 512*512 pixel
    Width          : DWORD;
    Height         : DWORD;
    BitsWidth      : BYTE;
    BitsHeight     : BYTE;
    constructor Create;
    procedure Free;
    destructor Destroy; override;
end;

////////////////////////////////////////////////////////////////////////////////

type
  TNormalTexture = Class(TObject)
    TextureName      : string;
    TexturePackage   : string;
    DetailTexture    : string;
    Palette          : string; //Name of used palette
    UsedPalette      : TPalette;
    UBits            : BYTE;                  //(obselote)
    VBits            : BYTE;                  //(obselote)
    USize            : DWORD;                 //(obselote)
    VSize            : DWORD;                 //(obselote)
    UClamp           : DWORD;                 //(obselote)
    VClamp           : DWORD;                 //(obselote)
    MipZero          : TUTColor;              //(obselote)
    MaxColor         : TUTColor;              //(obselote)
    InternalTime     : array [0..1] of DWORD; //(obselote)
    MipMapCount      : BYTE;
    MipMap           : TMipMap; //contains the actual Data and some properties
    B                : TBitmap; //used to create output
    MaxPreviewSize   : integer; //max. width or height of preview
    HasToBeResized   : Boolean; //if last available mipmap is larger than MaxPreviewSize, the mipmap has to be resized
    constructor Create;
    procedure   Free;
    destructor  Destroy; override;
  public
    procedure Load(const NameTable : TNameTable; ImportTable : TImportTable; ExportTable : TExportTable; PkgVer : integer; Name : string; MaxPreviewSize : integer; var F : File);
  private
    Offset           : DWORD;
    Size             : DWORD;
    procedure CreateBitmap;  //Creates the ouput-bitmap
end;
//////////////////////////////

implementation

////////////////////////////////////////////////////////////////////////////////
//TMipMap

constructor TMipMap.Create;
begin
end;

procedure TMipMap.Free;
begin
 if Self <> nil then Destroy;
end;

destructor TMipMap.Destroy;
begin
  inherited Destroy;
end;

////////////////////////////////////////////////////////////////////////////////
//TTexture

constructor TNormalTexture.Create;
begin
  MipZero  := TUTColor.Create;
  MaxColor := TUTColor.Create;
  MipMap   := TMipMap.Create;
  UsedPalette := TPalette.Create;
  B := TBitmap.Create;
end;

procedure TNormalTexture.Free;
begin
 if Self <> nil then Destroy;
end;

destructor TNormalTexture.Destroy;
begin
  MipZero.Free;
  MaxColor.Free;
  MipMap.Free;
  UsedPalette.Free;
  B.Free;

  inherited Destroy;
end;

procedure TNormalTexture.Load;
var
  PropertyName : DWORD;
  TypeByte     : BYTE;
  ArrayIndex   : DWORD;
  MipMapRead   : Boolean;
  MipMapsPassed: integer;
  //temp-stuff
  tempBYTE     : BYTE;
  tempDWORD    : DWORD;
  i            : integer;
begin
  //Get basic infos of texture (i.e. offset, size and actual name)
  for i:=0 to ExportTable.Count-1 do
    if (ExportTable.ExportClass.Strings[i] = 'Texture') then
      if (Name = ExportTable.ObjectName.Strings[i]) then
      begin
      Offset := StrToInt(ExportTable.SerialOffset.Strings[i]);
      Size := StrToInt(ExportTable.SerialSize.Strings[i]);
      TextureName := ExportTable.ObjectName.Strings[i];
      TexturePackage := ExportTable.ExportPackage.Strings[i];

      break;
    end;

  ///////////////////////////////////////////////////////////
  //Decode properties (entries must be known, order is unimportant), needed for a) palette and b) to find the actual data
  Seek(F, Offset);
  repeat
    PropertyName := ReadINDEX(F);

     //bUnlit
     if (NameTable.ObjectName.Strings[PropertyName] = 'bUnlit') then
     begin
       BlockRead(F, TypeByte, 1); //0x53 = FALSE, 0xD3 = TRUE
       BlockRead(F, tempByte, 1); //always 0x00
     end;

     //Friction
     if (NameTable.ObjectName[PropertyName] = 'Friction') then
     begin
       BlockRead(F, TypeByte, 1); //Should be 0x24 (Float)
       BlockRead(F, tempDWORD, 4);
     end;

     //Diffuse
     if (NameTable.ObjectName[PropertyName] = 'Diffuse') then
     begin
       BlockRead(F, TypeByte, 1); //Should be 0x24 (Float)
       BlockRead(F, tempDWORD, 4);
     end;

     //bWaterWavy
     if (NameTable.ObjectName.Strings[PropertyName] = 'bWaterWavy') then
     begin
       BlockRead(F, TypeByte, 1); //0x53 = FALSE, 0xD3 = TRUE
       BlockRead(F, tempByte, 1); //always 0x00
     end;

     //DrawScale (detailtextures)
     if (NameTable.ObjectName[PropertyName] = 'DrawScale') then
     begin
       BlockRead(F, TypeByte, 1); //Should be 0x24 (Float)
       BlockRead(F, tempDWORD, 4);
     end;

     //bHasComp
     if (NameTable.ObjectName[PropertyName] = 'bHasComp') then
     begin
       BlockRead(F, TypeByte, 1); //0x53 = FALSE, 0xD3 = TRUE
       BlockRead(F, tempDWORD, 4); //always 0x00
     end;
     //CompFormat
     if (NameTable.ObjectName.Strings[PropertyName] = 'CompFormat') then
     begin
       BlockRead(F, TypeByte, 1); //Should be 0x01 (BYTE value follows)
       BlockRead(F, tempBYTE, 1);
     end;

     //AnimNext
     if (NameTable.ObjectName[PropertyName] = 'AnimNext') then
     begin
       BlockRead(F, TypeByte, 1); //Should be 0x05 or 0x15 (ObjRef)
       ReadINDEX(F);
     end;
     //MaxFrameRate
     if (NameTable.ObjectName[PropertyName] = 'MaxFrameRate') then
     begin
       BlockRead(F, TypeByte, 1); //Should be 0x24 (Float)
       BlockRead(F, tempDWORD, 4);
     end;
     //MinFrameRate
     if (NameTable.ObjectName[PropertyName] = 'MinFrameRate') then
     begin
       BlockRead(F, TypeByte, 1); //Should be 0x24 (Float)
       BlockRead(F, tempDWORD, 4);
     end;

     //LODSet (unimportant for previews)
     if (NameTable.ObjectName.Strings[PropertyName] = 'LODSet') then
     begin
       BlockRead(F, TypeByte, 1); //Should be 0x01 (BYTE value follows)
       BlockRead(F, tempBYTE, 1);
     end;

     //bMasked
     if (NameTable.ObjectName.Strings[PropertyName] = 'bMasked') then
     begin
       BlockRead(F, TypeByte, 1); //0x53 = FALSE, 0xD3 = TRUE
       BlockRead(F, tempByte, 1); //always 0x00
     end;

     //DetailTexture
     if (NameTable.ObjectName.Strings[PropertyName] = 'DetailTexture') then
     begin
       BlockRead(F, TypeByte, 1); //Should be 0x05 (=ObjectRef)
       DetailTexture := GetObjectReference(NameTable, ImportTable, ExportTable, ReadINDEX(F));
     end;

     //Palette
     if (NameTable.ObjectName.Strings[PropertyName] = 'Palette') then
     begin
       BlockRead(F, TypeByte, 1); //Should be 0x05 or 0x15 (=ObjectReference)
       Palette := GetObjectReference(NameTable, ImportTable, ExportTable, ReadINDEX(F));
     end;

     //UBits
     if (NameTable.ObjectName.Strings[PropertyName] = 'UBits') then
     begin
       BlockRead(F, TypeByte, 1); //Should be 0x01 (BYTE value follows)
       BlockRead(F, UBits, 1)
     end;
     //VBits
     if (NameTable.ObjectName.Strings[PropertyName] = 'VBits') then
     begin
       BlockRead(F, TypeByte, 1); //Should be 0x01 (BYTE value follows)
       BlockRead(F, VBits, 1);
     end;

     //USize
     if (NameTable.ObjectName.Strings[PropertyName] = 'USize') then
     begin
       BlockRead(F, TypeByte, 1); //Should be 0x22 (DWORD-Integer value follows)
       BlockRead(F, USize, 4);
     end;
     //VSize
     if (NameTable.ObjectName.Strings[PropertyName] = 'VSize') then
     begin
       BlockRead(F, TypeByte, 1); //Should 0x22 (DWORD-Integer value follows)
       BlockRead(F, VSize, 4);
     end;

     //UClamp
     if (NameTable.ObjectName.Strings[PropertyName] = 'UClamp') then
     begin
       BlockRead(F, TypeByte, 1); //Should be 0x22 (DWORD-Integer value follows)
       BlockRead(F, UClamp, 4);
     end;
     //VClamp
     if (NameTable.ObjectName.Strings[PropertyName] = 'VClamp') then
     begin
       BlockRead(F, TypeByte, 1); //Should be 0x22 (DWORD-Integer follows)
       BlockRead(F, VClamp, 4);
     end;

     //MipZero
     if (NameTable.ObjectName.Strings[PropertyName] = 'MipZero') then
     begin
       BlockRead(F, TypeByte, 1); //Should be 0x2A (struct follows)
       if (NameTable.ObjectName.Strings[ReadINDEX(F)] = 'Color') then
       begin
         BlockRead(F, MipZero.r, 1);
         BlockRead(F, MipZero.g, 1);
         BlockRead(F, MipZero.b, 1);
         BlockRead(F, MipZero.a, 1);
       end;
     end;

     //MaxColor
     if (NameTable.ObjectName.Strings[PropertyName] = 'MaxColor') then
     begin
       BlockRead(F, TypeByte, 1); //Should be 0x2A (struct follows)
       if (NameTable.ObjectName.Strings[ReadINDEX(F)] = 'Color') then
       begin
         BlockRead(F, MaxColor.r, 1);
         BlockRead(F, MaxColor.g, 1);
         BlockRead(F, MaxColor.b, 1);
         BlockRead(F, MaxColor.a, 1);
       end;
     end;

     //InternalTime
     if (NameTable.ObjectName.Strings[PropertyName] = 'InternalTime') then
     begin
       BlockRead(F, TypeByte, 1); //important as InternalTim is often an array
       if (TypeByte = $22) then //No array or first entry in array
         BlockRead(F, InternalTime[0], 4)
       else
         if (TypeByte = $A2) then //array
         begin
           ArrayIndex := ReadINDEX(F);
           BlockRead(F, InternalTime[ArrayIndex],4);
         end;
     end;
  until (NameTable.ObjectName.Strings[PropertyName] = 'None');

  ///////////////////////////////////////////////////////////
  //Read appropiate mipmap (i.e. max 128 pixels width/height)
  MipMapRead := FALSE;                         //No mipmap read yet
  MipMapsPassed := 0;                          //no mipmaps passed yet
  BlockRead(F, MipMapCount, 1);        //MipMap-Count
  repeat
    if (USize > VSize) then  //x >= y
    begin
      if (PkgVer >= 63) then                      //PackageVersion 63 or higher
      begin
        BlockRead(F, MipMap.WidthOffset, 4);
        MipMap.MipMapSize := ReadINDEX(F);

        Seek(F, MipMap.WidthOffset);          //Skip MipMapData

        BlockRead(F, MipMap.Width, 4);
        BlockRead(F, MipMap.Height, 4);
        BlockRead(F, MipMap.BitsWidth, 1);
        BlockRead(F, MipMap.BitsWidth, 1);

        if (MipMap.Width <= MaxPreviewSize) then
        begin
          Seek(F, MipMap.WidthOffset-MipMap.MipMapSize);
          BlockRead(F, MipMap.MipMapData, MipMap.MipMapSize);
          Seek(F, FilePos(F)+10);
          MipMapRead := TRUE;                        //MipMap is now read
          HasToBeResized := FALSE;
        end
        else
          if (MipMapsPassed = MipMapCount-1) then
          begin
            Seek(F, MipMap.WidthOffset-MipMap.MipMapSize);
            BlockRead(F, MipMap.MipMapData, MipMap.MipMapSize);
            Seek(F, FilePos(F)+10);
            MipMapRead := TRUE;                        //MipMap is now read
            HasToBeResized := TRUE;                    //mipmap is larger than required one
          end;
      end
      else                                           //PackageVersion lower than 63
      begin
        MipMap.MipMapSize := ReadINDEX(F);

        MipMap.WidthOffset := FilePos(F)+MipMap.MipMapSize; //generate missing offset
        Seek(F, FilePos(F)+MipMap.MipMapSize);             //Skip MipMapData

        BlockRead(F, MipMap.Width, 4);
        BlockRead(F, MipMap.Height, 4);
        BlockRead(F, MipMap.BitsWidth, 1);
        BlockRead(F, MipMap.BitsWidth, 1);

        if (MipMap.Width <= MaxPreviewSize) then
        begin
          Seek(F, FilePos(F)-MipMap.MipMapSize-10);
          BlockRead(F, MipMap.MipMapData, MipMap.MipMapSize);
          Seek(F, FilePos(F)+10);
          MipMapRead := TRUE;                        //MipMap is now read
          HasToBeResized := FALSE;
        end
        else
          if (MipMapsPassed = MipMapCount-1) then
          begin
            Seek(F, MipMap.WidthOffset-MipMap.MipMapSize);
            BlockRead(F, MipMap.MipMapData, MipMap.MipMapSize);
            Seek(F, FilePos(F)+10);
            MipMapRead := TRUE;                        //MipMap is now read
            HasToBeResized := TRUE;                    //mipmap is larger than required one
          end;
      end;
    end
    else                                                    //x < y
    begin
      if (PkgVer >= 63) then                                //PackageVersion 63 higher
      begin
        BlockRead(F, MipMap.WidthOffset, 4);
        MipMap.MipMapSize := ReadINDEX(F);

        Seek(F, MipMap.WidthOffset);          //Skip MipMapData

        BlockRead(F, MipMap.Width, 4);
        BlockRead(F, MipMap.Height, 4);
        BlockRead(F, MipMap.BitsWidth, 1);
        BlockRead(F, MipMap.BitsWidth, 1);

        if (MipMap.Height <= MaxPreviewSize) then
        begin
          Seek(F, MipMap.WidthOffset-MipMap.MipMapSize);
          BlockRead(F, MipMap.MipMapData, MipMap.MipMapSize);
          Seek(F, FilePos(F)+10);
          MipMapRead := TRUE;                        //MipMap is now read
          HasToBeResized := FALSE;
        end
        else
          if (MipMapsPassed = MipMapCount-1) then
          begin
            Seek(F, MipMap.WidthOffset-MipMap.MipMapSize);
            BlockRead(F, MipMap.MipMapData, MipMap.MipMapSize);
            Seek(F, FilePos(F)+10);
            MipMapRead := TRUE;                        //MipMap is now read
            HasToBeResized := TRUE;                    //mipmap is larger than required one
          end;
      end                                                      //PackageVersion lower than 63
      else
      begin
        MipMap.MipMapSize := ReadINDEX(F);

        MipMap.WidthOffset := FilePos(F)+MipMap.MipMapSize; //generate missing offset
        Seek(F, FilePos(F)+MipMap.MipMapSize);              //Skip MipMapData

        BlockRead(F, MipMap.Width, 4);
        BlockRead(F, MipMap.Height, 4);
        BlockRead(F, MipMap.BitsWidth, 1);
        BlockRead(F, MipMap.BitsWidth, 1);

        if (MipMap.Height <= MaxPreviewSize) then
        begin
          Seek(F, FilePos(F)-MipMap.MipMapSize-10);
          BlockRead(F, MipMap.MipMapData, MipMap.MipMapSize);
          Seek(F, FilePos(F)+10);
          MipMapRead := TRUE;                        //MipMap is now read
          HasToBeResized := FALSE;
        end
        else
          if (MipMapsPassed = MipMapCount-1) then
          begin
            Seek(F, MipMap.WidthOffset-MipMap.MipMapSize);
            BlockRead(F, MipMap.MipMapData, MipMap.MipMapSize);
            Seek(F, FilePos(F)+10);
            MipMapRead := TRUE;                        //MipMap is now read
            HasToBeResized := TRUE;                    //mipmap is larger than required one
          end;
      end;
    end;
    MipMapsPassed := MipMapsPassed + 1;
  until (MipMapRead = TRUE);

  ///////////////////////////////////////////////////////////
  //Read palette
  UsedPalette.Load(NameTable, ImportTable, ExportTable, F, Palette, TexturePackage);

  //Create bitmap
  CreateBitmap;
end;

procedure TNormalTexture.CreateBitmap;
var
  y,x          : integer;
  Row          : PByteArray;
begin
  //Initialize everything
  B.Free;
  B := TBitmap.Create;
  B.Height := MipMap.Height;
  B.Width  := MipMap.Width;
  B.PixelFormat := pf8bit;

  //Assign Palette
  if (UsedPalette.WinPalette <> 0) then
    B.Palette := UsedPalette.WinPalette;
  //Create bitmap
  for y:=0 to MipMap.Height-1 do
  begin
    Row := B.ScanLine[y];
    for x:=0 to MipMap.Width-1 do
      Row[x] := MipMap.MipMapData[y*MipMap.Width+x];
  end;
end;

////////////////////////////////////////////////////////////////////////////////


end.
