////////////////////////////////////////////////////////////////////////////////
//
// PkgAux-Unit:
//
// Includes: - procdures for reading INDEX and NAME
//           - TNameTable, TImportTable and TExportTable declarations
//           - TUTColor and TPalette declaration
//
// (c) 2000, by Jesco Topp
////////////////////////////////////////////////////////////////////////////////
unit PkgAux;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

////////////////////////////////////////////////////////////////////////////////
//functions to read INDEX and NAME

  function ReadINDEX(var F: file): DWORD;  //Read INDEX type
  function ReadNAME(var F: file; PkgVer : integer): string;  //Read NAME type


////////////////////////////////////////////////////////////////////////////////
//Headertable-objects and ObjectReference-function

type
  TNameTable = Class(TObject)
    ObjectName     : TStrings;
    ObjectFlags    : TStrings;
    Count          : integer;
    Offset         : integer;
    constructor Create;
    procedure Free;
    destructor Destroy; override;
end;

type
  TImportTable = Class(TObject)
    ClassPackage   : TStrings;
    ClassName      : TStrings;
    ImportPackage  : TStrings;
    ObjectName     : TStrings;
    Count          : integer;
    Offset         : integer;
    constructor Create;
    procedure Free;
    destructor Destroy; override;
end;

type
  TExportTable = Class(TObject)
    ExportClass  : TStrings;
    Super        : TStrings;
    ExportPackage: TStrings;
    ObjectName   : TStrings;
    ObjectFlags  : TStrings;
    SerialSize   : TStrings;
    SerialOffset : TStrings;
    Count        : integer;
    Offset       : integer;
    constructor Create;
    procedure Free;
    destructor Destroy; override;
end;

  function  GetObjectReference(const NameTable : TNameTable; ImportTable : TImportTable; ExportTable : TExportTable; Value : DWORD): string;

////////////////////////////////////////////////////////////////////////////////
//Palette and color object

type
  TUTColor = Class(TObject)
    r       : BYTE;
    g       : BYTE;
    b       : BYTE;
    a       : BYTE;
    constructor Create;
    procedure Free;
    destructor Destroy; override;
end;
type
  TPalette = Class(TObject)
    PaletteName    : string;                     //Name of the palette
    PaletteSize    : DWORD;                      //Number of entries
    PaletteOffset  : DWORD;                      //Offset
    Color          : array [0..255] of TUTColor; //unconverted palette-entries
    WinPalette     : HPalette;                   //converted HPALETTE
    constructor Create;
    procedure Free;
    destructor Destroy; override;
  public
    //Loads the under Name specified palette and converts it to a HPALETTE
    procedure Load(const NameTable : TNameTable; ImportTable : TImportTable; ExportTable : TExportTable; var F : File; Name : string; TexturePackage : string); //Read and convert the specified palette
end;
////////////////////////////////////////////////////////////////////////////////

implementation

////////////////////////////////////////////////////////////////////////////////
//functions to read INDEX and NAME

///////////////////////
//Read INDEX-Format
function ReadINDEX;
var
  B0,B1,B2,B3,B4 : BYTE;
  D0,D1,D2,D3,D4 : DWORD;
  INDEX          : DWORD;
  IsNegative     : Boolean;
begin
  B0 := 0;
  B1 := 0;
  B2 := 0;
  B3 := 0;
  B4 := 0;

  BlockRead(F, B0, 1);
  if (B0 > $80) then
  begin
    B0 := B0 - $80;
    IsNegative := TRUE;
  end
  else
    IsNegative := FALSE;

  //Read Following Bytes
  if (B0 >= $40) then
  begin
    B0 := B0 - $40;
    BlockRead(F, B1, 1);
    if (B1 >= $80) then
    begin
      B1 := B1 - $80;
      BlockRead(F, B2, 1);
      if (B2 >= $80) then
      begin
        B2 := B2 - $80;
        BlockRead(F, B3, 1);
        if (B3 >= $80) then
        begin
          B3 := B3 - $80;
          BlockRead(F, B4, 1);
        end;
      end;
    end;
  end;

  //Assign DWORDs
  D0 := B0;
  D1 := B1;
  D2 := B2;
  D3 := B3;
  D4 := B4;
  //Shift Bytes
  D1 := D1 shl 6;
  D2 := D2 shl 13;
  D3 := D3 shl 20;
  D4 := D4 shl 27;

  //Create resulting DWORD
  INDEX := D0 + D1 + D2 + D3 + D4;

  //Check for sign
  if (IsNegative = TRUE) then INDEX := INDEX * (-1);

  Result := INDEX;
end;

///////////////////////
//Read NAME-Format
function ReadNAME;
var
  zero : BYTE;
  i    : integer;
  NAME : string;
  temp : char;
begin
  i := 0;
  if (PkgVer < 64) then
  begin
    repeat
    i := i + 1;
    BlockRead(F, zero, 1);
    if (zero <> $00) then
    begin
      Seek(F, FilePos(F) - 1);
      BlockRead(F, temp, 1);
      Insert(temp, NAME, i);
    end;
    until (zero = $00);
  end
  else
  begin
    Seek(F, FilePos(F) + 1); //quit length byte
    repeat
    i := i + 1;
    BlockRead(F, zero, 1);
    if (zero <> $00) then
    begin
      Seek(F, FilePos(F) - 1);
      BlockRead(F, temp, 1);
      Insert(temp, NAME, i);
    end;
    until (zero = $00);
  end;

  Result := NAME;
end;

////////////////////////////////////////////////////////////////////////////////
//Headertable-objects

///////////////////////
//TNameTable
constructor TNameTable.Create;
begin
  ObjectName  := TStringList.Create;
  ObjectFlags := TStringList.Create;
end;

procedure TNameTable.Free;
begin
 if Self <> nil then Destroy;
end;

destructor TNameTable.Destroy;
begin
  ObjectName.Free;
  ObjectFlags.Free;

  inherited Destroy;
end;

///////////////////////
//TImportTable
constructor TImportTable.Create;
begin
    ClassPackage   := TStringList.Create;
    ClassName      := TStringList.Create;
    ImportPackage  := TStringList.Create;
    ObjectName     := TStringList.Create;
end;

procedure TImportTable.Free;
begin
 if Self <> nil then Destroy;
end;

destructor TImportTable.Destroy;
begin
  ClassPackage.Free;
  ClassName.Free;
  ImportPackage.Free;
  ObjectName.Free;

  inherited Destroy;
end;

///////////////////////
//TExportTable
constructor TExportTable.Create;
begin
    ExportClass  := TStringList.Create;
    Super        := TStringList.Create;
    ExportPackage:= TStringList.Create;
    ObjectName   := TStringList.Create;
    ObjectFlags  := TStringList.Create;
    SerialSize   := TStringList.Create;
    SerialOffset := TStringList.Create;
end;

procedure TExportTable.Free;
begin
 if Self <> nil then Destroy;
end;

destructor TExportTable.Destroy;
begin
  ExportClass.Free;
  Super.Free;
  ExportPackage.Free;
  ObjectName.Free;
  ObjectFlags.Free;
  SerialSize.Free;
  SerialOffset.Free;

  inherited Destroy;
end;

//Evaluate Object-Reference
function GetObjectReference;
begin
  if (Value = 0) then Result := '';
  if (Value < 0) then Result := ImportTable.ObjectName.Strings[-Value-1];
  if (Value > 0) then Result := ExportTable.ObjectName.Strings[Value-1];
end;

////////////////////////////////////////////////////////////////////////////////
//Palette and color object

///////////////////////
//TUTColor
constructor TUTColor.Create;
begin
end;

procedure TUTColor.Free;
begin
 if Self <> nil then Destroy;
end;

destructor TUTColor.Destroy;
begin
  inherited Destroy;
end;

///////////////////////
//TPalette (UTPalette AND WindowsPalette)
constructor TPalette.Create;
var
  i         : integer;
begin
  for i:=0 to 255 do
    Color[i] := TUTColor.Create;
end;

procedure TPalette.Free;
begin
 if Self <> nil then Destroy;
end;

destructor TPalette.Destroy;
var
  i         : integer;
begin
  for i:=0 to 255 do
    Color[i].Free;

  inherited Destroy;
end;

procedure TPalette.Load;
var
  i       : integer;
  pal     : PLogPalette;
begin
  PaletteOffset := 0;
  PaletteName := Name;

  //Get offset of palette
  //palette and texture are in the same package
  for i:=0 to ExportTable.Count-1 do
    if (PaletteName = ExportTable.ObjectName[i]) then
      if (ExportTable.ExportClass.Strings[i] = 'Palette') then
        if (ExportTable.ExportPackage.Strings[i] = TexturePackage) then
          PaletteOffset := StrToInt(ExportTable.SerialOffset[i]);

  //palette and texture are in different packages
  if (PaletteOffset = 0) then
    for i:=0 to ExportTable.Count-1 do
      if (PaletteName = ExportTable.ObjectName[i]) then
        if (ExportTable.ExportClass.Strings[i] = 'Palette') then
          PaletteOffset := StrToInt(ExportTable.SerialOffset[i]);

  Seek(F, PaletteOffset);

  //Decode properties (there should be only a none-index)
  ReadINDEX(F);

  //Read the palette
  PaletteSize := ReadINDEX(F);
  for i:=0 to PaletteSize-1 do
  begin
    BlockRead(F, Color[i].r, 1);
    BlockRead(F, Color[i].g, 1);
    BlockRead(F, Color[i].b, 1);
    BlockRead(F, Color[i].a, 1);
  end;

  //Create palette
  pal := nil;
  try
    GetMem(pal, sizeof(TLogPalette) + sizeof(TPaletteEntry) * PaletteSize);
    pal.palVersion := $300;
    pal.palNumEntries := PaletteSize;
    for i := 0 to PaletteSize-1 do
    begin
      pal.palPalEntry[i].peRed := Color[i].r;
      pal.palPalEntry[i].peGreen := Color[i].g;
      pal.palPalEntry[i].peBlue := Color[i].b;
      pal.palPalEntry[i].peFlags := 0;
    end;
    WinPalette := CreatePalette(pal^);
  finally
    FreeMem(pal);
  end;
end;

////////////////////////////////////////////////////////////////////////////////
end.
